import bitjws
import httpretty
import json
import os
import pytest
import time
from bravado.swagger_model import load_file
from bravado_bitjws.client import BitJWSSwaggerClient
from bravado_bitjws.requests_client import BitJWSAuthenticator

specurl = "%s/example/swagger.json" % os.path.realpath(os.path.dirname(__file__))
wif = "KweY4PozGhtkGPMvvD7vk7nLiN6211XZ2QGxLBMginAQW7MBbgp8"

def test_good_call():
    url = 'http://0.0.0.0:8002/coin'
    bjauth = BitJWSAuthenticator('0.0.0.0', privkey=wif)
    client = BitJWSSwaggerClient.from_spec(load_file(specurl),
                                           origin_url=url, privkey=wif)
    coin = [{'metal':'m', 'mint':'n'}]

    data = bitjws.sign_serialize(bjauth.privkey, requrl="/response",
                                 iat=time.time(),
                                 data=coin)

    httpretty.enable()
    httpretty.register_uri(httpretty.GET, url,
                           body=data, content_type = 'application/jose')

    resp = client.coin.findCoin().result()
    assert resp[0].metal == coin[0]['metal']
    assert resp[0].mint == coin[0]['mint']

    httpretty.disable()
    httpretty.reset()




# -*- coding: utf-8 -*-
"""
The :class:`SwaggerClient` provides an interface for making API calls based on
a swagger spec, and returns responses of python objects which build from the
API response.

Structure Diagram::

        +---------------------+
        |                     |
        |    SwaggerClient    |
        |                     |
        +------+--------------+
               |
               |  has many
               |
        +------v--------------+
        |                     |
        |     Resource        +------------------+
        |                     |                  |
        +------+--------------+         has many |
               |                                 |
               |  has many                       |
               |                                 |
        +------v--------------+           +------v--------------+
        |                     |           |                     |
        |     Operation       |           |    SwaggerModel     |
        |                     |           |                     |
        +------+--------------+           +---------------------+
               |
               |  uses
               |
        +------v--------------+
        |                     |
        |     HttpClient      |
        |                     |
        +---------------------+


To get a client

.. code-block:: python

    client = bravado.client.SwaggerClient.from_url(swagger_spec_url)
"""
import bitjws
import urlparse

from bravado.client import SwaggerClient

from bravado_bitjws.requests_client import BitJWSRequestsClient

__all__ = ['BitJWSSwaggerClient']


class BitJWSSwaggerClient(SwaggerClient):
    """
    A client for accessing a Swagger-documented RESTful service,
    which also uses bitjws authentication.
    """

    @classmethod
    def from_url(cls, spec_url, http_client=None, privkey=None, **kwargs):
        """
        Build a :class:`SwaggerClient` from a url to the Swagger
        specification for a RESTful API.

        :param spec_url: url pointing at the swagger API specification
        :type spec_url: str
        :param http_client: an HTTP client used to perform requests
        :type  http_client: :class:`bravado.http_client.HttpClient`
        """
        if privkey is None:
            privkey = bitjws.PrivateKey()
        elif isinstance(privkey, str):
            privkey = bitjws.PrivateKey(bitjws.wif_to_privkey(privkey))

        if http_client is None:
            host = urlparse.urlsplit(spec_url).hostname
            http_client = BitJWSRequestsClient()
            private_key = bitjws.privkey_to_wif(privkey.private_key)
            http_client.set_bitjws_key(host, private_key)
        return SwaggerClient.from_url(spec_url, http_client=http_client,
                                      **kwargs)

    @classmethod
    def from_spec(cls, spec_dict, origin_url=None, http_client=None,
                  privkey=None, **kwargs):
        """
        Build a :class:`SwaggerClient` from swagger api docs

        :param spec_dict: a dict with a Swagger spec in json-like form
        :param origin_url: the url used to retrieve the spec_dict
        :type  origin_url: str
        :param http_client: an HTTP client used to perform requests
        :type  http_client: :class:`bravado.http_client.HttpClient`
        :param str privkey: The WIF private key to use for bitjws signing
        """
        if privkey is None:
            privkey = bitjws.PrivateKey()
        elif isinstance(privkey, str):
            privkey = bitjws.PrivateKey(bitjws.wif_to_privkey(privkey))

        if http_client is None:
            host = urlparse.urlsplit(origin_url).hostname
            http_client = BitJWSRequestsClient()
            private_key = bitjws.privkey_to_wif(privkey.private_key)
            http_client.set_bitjws_key(host, private_key)

        return SwaggerClient.from_spec(spec_dict, http_client=http_client,
                                       **kwargs)




import bitjws
import json
import os
import pytest
import requests
import time
from bravado.swagger_model import load_file
from bravado_bitjws.client import BitJWSSwaggerClient
from bravado_bitjws.requests_client import *

specurl = "%s/example/swagger.json" % os.path.realpath(os.path.dirname(__file__))
wif = "KweY4PozGhtkGPMvvD7vk7nLiN6211XZ2QGxLBMginAQW7MBbgp8"


def test_apply_auth():
    url = 'http://0.0.0.0:8002/path/to/api'
    bjauth = BitJWSAuthenticator('0.0.0.0', privkey=wif)
    data = ""
    headers = {}
    params = {'message': 'goes here for sure'}
    request = requests.Request(method='GET', url=url, headers=headers,
                               data=data, params=params)
    req = bjauth.apply(request)
    assert len(req.params) == 0
    assert req.headers['content-type'] == 'application/jose'

    h, p = bitjws.validate_deserialize(req.data, requrl=url)
    assert 'message' in p['data']
    assert p['data']['message'] == params['message']
    assert 'aud' in p
    assert p['aud'] == url


def test_apply_auth_json():
    url = 'http://0.0.0.0:8002/path/to/api'
    bjauth = BitJWSAuthenticator('0.0.0.0', privkey=wif)
    rawdata = {'message': 'goes here for sure'}
    data = json.dumps(rawdata)
    headers = {'content-type': 'application/json'}
    params = {'mess': 'couldgohere'}
    request = requests.Request(method='GET', url=url, headers=headers,
                               data=data, params=params)
    req = bjauth.apply(request)
    assert len(req.params) == 0
    assert req.headers['content-type'] == 'application/jose'

    h, p = bitjws.validate_deserialize(req.data, requrl=url)
    assert 'message' in p['data']
    assert p['data']['message'] == rawdata['message']


def test_response_good():
    bjauth = BitJWSAuthenticator('0.0.0.0', privkey=wif)

    headers = {'content-type': 'application/jose'}
    params = {'mess': 'goeshere'}

    data = bitjws.sign_serialize(bjauth.privkey, requrl="/response",
                                 iat=time.time(),
                                 data=params)

    resp = requests.models.Response()
    resp.status_code = 200
    resp._content = b'%s' % data
    resp.headers = headers

    response = BitJWSRequestsResponseAdapter(resp)
    assert response.json() == params


def test_response_json():
    bjauth = BitJWSAuthenticator('0.0.0.0', privkey=wif)

    headers = {'content-type': 'application/json'}
    params = {'mess': 'goeshere'}

    data = json.dumps(params)

    resp = requests.models.Response()
    resp.status_code = 200
    resp._content = b'%s' % data
    resp.headers = headers

    response = BitJWSRequestsResponseAdapter(resp)
    assert response.json() == params


def test_response_bad_requrl():
    bjauth = BitJWSAuthenticator('0.0.0.0', privkey=wif)

    headers = {'content-type': 'application/jose'}
    params = {'mess': 'couldgohere'}

    data = bitjws.sign_serialize(bjauth.privkey, requrl="/request/not/response",
                                 iat=time.time(),
                                 data=params)

    resp = requests.models.Response()
    resp.status_code = 200
    resp._content = b'%s' % data
    resp.headers = headers

    response = BitJWSRequestsResponseAdapter(resp)
    pytest.raises(bitjws.jws.InvalidPayload, response.json)




import bitjws
import httpretty
import json
import os
import pytest
from bravado.swagger_model import load_file
from bravado_bitjws.client import BitJWSSwaggerClient

specurl = "%s/example/swagger.json" % os.path.realpath(os.path.dirname(__file__))
wif = "KweY4PozGhtkGPMvvD7vk7nLiN6211XZ2QGxLBMginAQW7MBbgp8"

def test_init_WIF():
    client = BitJWSSwaggerClient.from_spec(load_file(specurl),
                                           'https://0.0.0.0:8002', privkey=wif)
    assert 'coin' in client.swagger_spec.definitions
    assert 'user' in client.swagger_spec.definitions
    ckey = client.coin.addCoin.swagger_spec.http_client.authenticator.privkey
    assert isinstance(ckey, bitjws.PrivateKey)
    assert wif == bitjws.privkey_to_wif(ckey.private_key)


def test_init_PrivateKey():
    privkey = bitjws.PrivateKey(bitjws.wif_to_privkey(wif))

    client = BitJWSSwaggerClient.from_spec(load_file(specurl),
                                           'https://0.0.0.0:8002',
                                           privkey=privkey)
    assert 'coin' in client.swagger_spec.definitions
    assert 'user' in client.swagger_spec.definitions
    ckey = client.coin.addCoin.swagger_spec.http_client.authenticator.privkey
    assert isinstance(ckey, bitjws.PrivateKey)
    assert bitjws.privkey_to_wif(privkey.private_key) == \
            bitjws.privkey_to_wif(ckey.private_key)

def test_init():
    client = BitJWSSwaggerClient.from_spec(load_file(specurl),
                                           'https://0.0.0.0:8002')
    assert 'coin' in client.swagger_spec.definitions
    assert 'user' in client.swagger_spec.definitions
    ckey = client.coin.addCoin.swagger_spec.http_client.authenticator.privkey
    assert isinstance(ckey, bitjws.PrivateKey)

def test_init_url_WIF():
    url = "http://spec.com/swagger.json"
    spec = load_file(specurl)
    httpretty.enable()
    httpretty.register_uri(httpretty.GET, url,
                           body=json.dumps(spec))

    client = BitJWSSwaggerClient.from_url(url, privkey=wif)
    assert 'coin' in client.swagger_spec.definitions
    assert 'user' in client.swagger_spec.definitions
    ckey = client.coin.addCoin.swagger_spec.http_client.authenticator.privkey
    assert isinstance(ckey, bitjws.PrivateKey)
    assert wif == bitjws.privkey_to_wif(ckey.private_key)

    httpretty.disable()
    httpretty.reset()

def test_init_url_PrivateKey():
    privkey = bitjws.PrivateKey(bitjws.wif_to_privkey(wif))

    url = "http://spec.com/swagger.json"
    spec = load_file(specurl)
    httpretty.enable()
    httpretty.register_uri(httpretty.GET, url,
                           body=json.dumps(spec))

    client = BitJWSSwaggerClient.from_url(url, privkey=privkey)
    assert 'coin' in client.swagger_spec.definitions
    assert 'user' in client.swagger_spec.definitions
    ckey = client.coin.addCoin.swagger_spec.http_client.authenticator.privkey
    assert isinstance(ckey, bitjws.PrivateKey)
    assert bitjws.privkey_to_wif(privkey.private_key) == bitjws.privkey_to_wif(ckey.private_key)

    httpretty.disable()
    httpretty.reset()




from setuptools import setup
import bravado_bitjws

classifiers = [
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 2",
    "Topic :: Software Development :: Libraries",
]

setup(
    name="bravado-bitjws",
    version='0.1.2',
    description="Library for accessing Swagger-enabled API's with bitjws authentication.",
    author='Deginner',
    author_email='ira@deginner.com',
    url="https://github.com/deginner/bravado-bitjws",
    license='BSD',
    classifiers=classifiers,
    include_package_data=True,
    packages=["bravado_bitjws"],
    setup_requires=['pytest-runner'],
    install_requires=[
        "bravado-core >= 3.0.2",
        "bravado >= 7.0.0",
        "pyasn1 >= 0.1.9",
        "secp256k1==0.11",
        "bitjws==0.6.3.1"
    ],
    tests_require=[
        "httpretty",
        "pytest",
        "pytest-cov"
    ]
)



# -*- coding: utf-8 -*-
"""
Wrappers for the bravado.requests_client classes.
"""

import bitjws
import json
import requests
import requests.auth
import time
from bravado.requests_client import (Authenticator, RequestsClient,
                                     RequestsResponseAdapter,
                                     RequestsFutureAdapter)

from bravado.http_future import HttpFuture

__all__ = ['BitJWSRequestsClient', 'BitJWSAuthenticator',
           'BitJWSRequestsResponseAdapter']


class BitJWSAuthenticator(Authenticator):
    """BitJWS authenticator uses JWS and the CUSTOM-BITCOIN-SIGN algorithm.

    :param host: Host to authenticate for.
    :param privkey: Private key as a WIF string
    """

    def __init__(self, host, privkey):
        super(BitJWSAuthenticator, self).__init__(host)
        self.privkey = bitjws.PrivateKey(bitjws.wif_to_privkey(privkey))

    def apply(self, request):
        if len(request.data) > 0:
            data = bitjws.sign_serialize(self.privkey, requrl=request.url,
                                         iat=time.time(),
                                         data=json.loads(request.data))
        else:
            data = bitjws.sign_serialize(self.privkey, requrl=request.url,
                                         iat=time.time(),
                                         data=request.params)
        request.params = {}
        request.data = data
        request.headers['content-type'] = 'application/jose'
        return request


class BitJWSRequestsClient(RequestsClient):
    """Synchronous HTTP client implementation.
    """

    def __init__(self):
        super(BitJWSRequestsClient, self).__init__()
        self.session = requests.Session()
        self.authenticator = None

    def request(self, request_params, operation=None, response_callbacks=None,
                also_return_response=False):
        """
        :param request_params: complete request data.
        :type request_params: dict
        :param operation: operation that this http request is for. Defaults
            to None - in which case, we're obviously just retrieving a Swagger
            Spec.
        :type operation: :class:`bravado_core.operation.Operation`
        :param response_callbacks: List of callables to post-process the
            incoming response. Expects args incoming_response and operation.
        :param also_return_response: Consult the constructor documentation for
            :class:`bravado.http_future.HttpFuture`.

        :returns: HTTP Future object
        :rtype: :class: `bravado_core.http_future.HttpFuture`
        """
        sanitized_params, misc_options = self.separate_params(request_params)

        requests_future = RequestsFutureAdapter(
            self.session,
            self.authenticated_request(sanitized_params),
            misc_options)

        return HttpFuture(
            requests_future,
            BitJWSRequestsResponseAdapter,
            operation,
            response_callbacks,
            also_return_response)


    def set_bitjws_key(self, host, privkey):
        """
        :param host: The host to authenticate for.
        :param privkey: This client's private key to sign with.
        """
        self.authenticator = BitJWSAuthenticator(host=host, privkey=privkey)


class BitJWSRequestsResponseAdapter(RequestsResponseAdapter):
    """Wraps a requests.models.Response object to provide a uniform interface
    to the response innards.
    """

    def json(self, **kwargs):
        jso = {}
        if 'content-type' in self._delegate.headers and \
                'application/jose' in self._delegate.headers['content-type']:
            rawtext = self.text.decode('utf8')
            headers, jwtpayload = \
                    bitjws.validate_deserialize(rawtext, requrl='/response')
            if headers is None:
                raise bitjws.jws.InvalidPayload("Response failed validation")
            if 'data' in jwtpayload:
                jso = jwtpayload['data']
            else:
                jso = jwtpayload
        else:
            jso = self._delegate.json(**kwargs)
        return jso

